#! /usr/bin/env python

import unittest, os, re
import shutil, tempfile

import base, util


class TarballerTestCase(base.TestCase):
	def setUp(self):
		self.tarballer = util.Tarballer('foo')
		self.tmpdir = tempfile.mktemp()
		self.testdir = tempfile.mktemp()
		os.mkdir(self.tmpdir, 0700)
		os.mkdir(self.testdir, 0700)
	def tearDown(self):
		if hasattr(self,'tmpdir') and os.path.exists(self.tmpdir):
			print 'removing',self.tmpdir
			shutil.rmtree(self.tmpdir)
		if hasattr(self,'testdir') and os.path.exists(self.testdir):
			print 'removing',self.testdir
			shutil.rmtree(self.testdir)
		del self.tarballer

	def do_addver(self, *args, **kw):
		self.tarballer.addver(*args, **kw)
	def do_unpack(self, ver):
		olddir = os.getcwd()
		os.chdir(self.tmpdir)
		try:
			return os.path.join(self.tmpdir,util.unpack(self.tarballer.tarpath(ver)))
		finally:
			os.chdir(olddir)
	
	def test_existence(self):
		self.do_addver("1.0", add=["dir1/"])
		self.do_addver("1.1", add=["dir1/file1"])
		self.do_addver("1.2", rm=["dir1/file1"])
		self.do_addver("1.3", rm=["dir1/"])
		# empty dir:
		self.failUnless(self.tarballer.verify("1.0", self.testdir))
		self.failUnless(self.tarballer.verify("1.1", self.testdir))
		self.failUnless(self.tarballer.verify("1.2", self.testdir))
		self.vfailIf(self.tarballer.verify("1.3", self.testdir))
		# dir1 exists:
		os.mkdir(os.path.join(self.testdir, "dir1"))
		self.vfailIf(self.tarballer.verify("1.0", self.testdir))
		self.failUnless(self.tarballer.verify("1.1", self.testdir))
		self.vfailIf(self.tarballer.verify("1.2", self.testdir))
		self.failUnless(self.tarballer.verify("1.3", self.testdir))
		# dir1 and dir1/file1 exist:
		d = self.do_unpack("1.1")
		shutil.copyfile(os.path.join(d,"dir1","file1"), os.path.join(self.testdir, "dir1","file1"))
		self.failUnless(self.tarballer.verify("1.0", self.testdir))
		self.vfailIf(self.tarballer.verify("1.1", self.testdir))
		self.failUnless(self.tarballer.verify("1.2", self.testdir))
		self.failUnless(self.tarballer.verify("1.3", self.testdir))

	def test_different_file(self):
		self.do_addver("1.0", add=["file1","file2"])
		d = self.do_unpack("1.0")
		shutil.copyfile(os.path.join(d,"file1"), os.path.join(self.testdir, "file2"))
		shutil.copyfile(os.path.join(d,"file2"), os.path.join(self.testdir, "file1"))
		self.failUnless(self.tarballer.verify("1.0", self.testdir))

	def test_different_file2(self):
		self.do_addver("1.0", add=["file1","dir1/file1"])
		d = self.do_unpack("1.0")
		os.mkdir(os.path.join(self.testdir, "dir1"))
		shutil.copyfile(os.path.join(d, "file1"), os.path.join(self.testdir, "dir1", "file1"))
		shutil.copyfile(os.path.join(d, "dir1", "file1"), os.path.join(self.testdir, "file1"))
		self.failUnless(self.tarballer.verify("1.0", self.testdir))

	def test_different_version_file(self):
		self.do_addver("1.0", add=["file1"])
		self.do_addver("1.1", add=["file1"])
		d = self.do_unpack("1.1")
		shutil.copyfile(os.path.join(d,"file1"), os.path.join(self.testdir, "file1"))
		self.failUnless(self.tarballer.verify("1.0", self.testdir))
		d = self.do_unpack("1.0")
		shutil.copyfile(os.path.join(d,"file1"), os.path.join(self.testdir, "file1"))
		self.failUnless(self.tarballer.verify("1.1", self.testdir))


if __name__ == '__main__':
	unittest.main()

